/****************************************************************************
 * UniPROG Universal programmer 
 * Device algoritms
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 06/06/2001 - First version, Atmel algoritms only
 *              fixed to eep programmer
 * 07/09/2003 - Ported to new multi-programmer architecture
 *
 */

#include "stdhdr.h"
#include "progdev.h"
#include "portio.h"
#include "algoritms.h"
#include "chipdb.h"

// Generic Algoritms (read only)

// Read a block from memory
int genReadBlock(unsigned char *buf, unsigned int size, long addr) {
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  assert(buf);

   while (size--)
      *buf++ = pd->read_byte_addr(pd, addr++);

  return AL_NOERR;
  }


/* Power on Vcc */
int genVccOn( void ) {
  struct prog_dev *pd = prgdev_get_selected();
  int err;
  assert(pd);

  err = pd->command(CMD_VCC, NULL, 1, 0);
  assert( err == ERR_OK);
  return err;
}


/* Power off Vcc */
int genVccOff( void ) {
  struct prog_dev *pd = prgdev_get_selected();
  int err;
  assert(pd);

  err = pd->command(CMD_VCC, NULL, 0, 0);
  assert( err == ERR_OK);
  return err;
}


// Atmel 29x Functions

// Get ID data
int atGetID( unsigned int *ID ) {
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  assert(ID);

  // Softwate ID entry
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0x90);
  waitms(20);

  // Read ID
  *ID =  pd->read_byte_addr(pd, 0x0001) ;
  *ID |= pd->read_byte_addr(pd, 0x0000) << 8;
  //*ID |= pd->read_byte_addr(pd, 0x0003) << 16;
  //*ID |= pd->read_byte_addr(pd, 0x0002) << 24;

  // Softwate ID exit
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0xF0);

  return AL_NOERR;
  }

int atErase( void ) {
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0x80);
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0x10);
  waitms(20);
  return 0;
  }

// program a entire sector to memory
int ramWriteSector(unsigned char *buf, unsigned int size, long addr, CHIPREG *chip) {
  unsigned int sz_sector = (chip->size * 1024) / chip->nSectors;
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  assert(buf);

  // check that loaded data is equal or smaller than a single sector
  assert(size <= sz_sector);
  // check if address doesnt bounds out of the chip size
  assert(addr+size <= chip->size * 1024);

  // load the entire sector to the memory, this must be FAST
  while (size--)
    pd->write_byte_addr(pd, addr++, *buf++);

  // at this point, the sector is writed, but must be verified
  return AL_NOERR;
  }



// program a entire sector to memory
int atWriteSector(unsigned char *buf, unsigned int size, long addr, CHIPREG *chip) {
  DWORD twc;                    // NOTE: Win32 specific
  unsigned char b1;
  unsigned int sz_sector = (chip->size * 1024) / chip->nSectors;
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  assert(buf);

  // check that loaded data is equal or smaller than a single sector
  assert(size <= sz_sector);
  // check if address doesnt bounds out of the chip size
  assert(addr+size <= chip->size * 1024);

  // SDP sequence (unlocks write protection for Atmel devices)
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0xA0);

  // load the entire sector to the memory, this must be FAST
  while (size--)
    pd->write_byte_addr(pd, addr++, *buf++);

  // start the twc timer
  twc = getms();

  // Polling loop
  buf--;
  addr--;
  do {
     if ((getms() - twc) > 100) {   // sector must write in less than 20ms
       //       printf("Sector %04lX, Timetout Error", addr);
       return AL_ERRTIMEOUT;
     }
     b1 = pd->read_byte_addr(pd, addr); // get bit toggle
     //     printf("%02X != %02X\n", b1, *buf);
  } while (b1 != *buf); 

  //printf("Sector %04lX, written in %ld ms.\n", addr, GetTickCount() - twc);

  // at this point, the sector is writed, but must be verified
  return AL_NOERR;
  }


// check if a sector contains valid data
int atVerifySector(unsigned char *buf, unsigned int size, long addr, CHIPREG *chip) {
  unsigned int sz_sector;  
  unsigned char c1, c2;
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);
  assert(buf);

  if (chip->nSectors)
    sz_sector = (chip->size * 1024) / chip->nSectors;
  else
    sz_sector = size;

  // check that loaded data is smaller than a single sector
  assert(size <= sz_sector);

  // Check sector is equal than the memory
  while (size--)
    {
    c1 = pd->read_byte_addr(pd, addr++);
    c2 = *buf++;
    if (c2 != c1)
      {
      buf--;
      addr--;
      //printf("\nVerify error at %04lX, %02X <> %02X\n", addr, c2, c1);
      return AL_ERRVERIFY;
      }
    }
  // at this point, the sector is correctly verified
  return AL_NOERR;
  }

/* AMD Flash algorithms */

// Get ID data
int amGetSectorProtection( void ) {
  struct prog_dev *pd = prgdev_get_selected();
  int sec, id, i;
  assert(pd);

  // Softwate ID entry
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0x90);
  waitms(20);

  // Read ID
  id =  pd->read_byte_addr(pd, 0x0001) ;
  id |= pd->read_byte_addr(pd, 0x0000) << 8;

  /* am28F040, 8 sectors */
  if (id == 0x01A4) {
    for (i = 0; i < 8; i++) {
      sec = pd->read_byte_addr(pd, i << 16 | 2);
      printf("  Sector %d (%05Xh-%05Xh): %02X (%s)\n", i,
	     (i << 16), 
	     ((i+1) << 16) - 1,
	     sec, 
	     (sec & 1) ? "protected" : "unprotected");
    }  
  }

  // Softwate ID exit
  pd->write_byte_addr(pd, 0x5555, 0xAA);
  pd->write_byte_addr(pd, 0x2AAA, 0x55);
  pd->write_byte_addr(pd, 0x5555, 0xF0);

  return AL_NOERR;
  }




/* Check hardware */
int CheckHardware( void ) {
  struct prog_dev *pd = prgdev_get_selected();
  assert(pd);

  return pd->test_hardware(pd, 0);
}
